# Copyright (c) 2022,2023,2024 Kian-Meng Ang

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Generals Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""Nox configuration."""

import ast
import datetime

import nox
from packaging.version import Version


@nox.session()
def deps(session: nox.Session) -> None:
    """Update pre-commit hooks and deps."""
    session.install("pre-commit", "pipenv")
    session.run("pre-commit", "autoupdate", *session.posargs)
    session.run("pipenv", "update", env={"PIPENV_VERBOSITY": "-1"})


@nox.session(python="3.12")
def lint(session: nox.Session) -> None:
    """Run pre-commit tasks.

    For running selected task within pre-commit:

        nox -s lint -- pylint
    """
    _pipenv_install(session)
    session.run("pre-commit", "run", "--all-files", *session.posargs)


@nox.session(python=["3.8", "3.9", "3.10", "3.11", "3.12"], reuse_venv=True)
def test(session: nox.Session) -> None:
    """Run test."""
    _pipenv_install(session)
    session.run("pytest", "--numprocesses", "auto", *session.posargs)


@nox.session(python="3.12")
def cov(session: nox.Session) -> None:
    """Run test coverage."""
    _pipenv_install(session)
    session.run(
        "pytest",
        "--numprocesses",
        "auto",
        "--cov",
        "--cov-report=term",
        "--cov-report=html",
        *session.posargs,
    )


@nox.session(python="3.8")
def doc(session: nox.Session) -> None:
    """Build doc with sphinx."""
    _pipenv_install(session)
    session.run(
        "sphinx-build",
        "docs/source/",
        "docs/build/html",
        *session.posargs,
    )


@nox.session(python="3.12", reuse_venv=True)
def readme(session: nox.Session) -> None:
    """Update console help menu to readme."""
    _pipenv_install(session)
    with open("README.md", "r+", encoding="utf8") as f:
        help_message = session.run("heatmap", "-h", "-vvv", silent=True)
        help_codeblock = f"\n\n```console\n{help_message}```\n\n"

        content = f.read()
        marker = content.split("<!--help !-->")[1]
        readme_md = content.replace(marker, help_codeblock)

        f.seek(0)
        f.write(readme_md)
        f.truncate()


@nox.session(python="3.12", reuse_venv=True)
def release(_session: nox.Session) -> None:
    """Bump release."""
    with open("heatmap_cli/__init__.py", "r", encoding="utf8") as f:
        tree = ast.parse(f.read())
        current_version = None
        for node in ast.walk(tree):
            if isinstance(node, ast.Assign) and len(node.targets) == 1:
                target, value = node.targets[0], node.value
                if target.id == "__version__" and isinstance(  # type: ignore
                    value, ast.Str
                ):
                    current_version = value.s
                    break

        if current_version is None:
            raise ValueError("Missing __version__ variable in __init__.py")

        before_version = Version(current_version)
        after_version = (
            f"{before_version.major}."
            f"{before_version.minor}."
            f"{before_version.micro + 1}"
        )
        _search_and_replace(
            "heatmap_cli/__init__.py", str(before_version), after_version
        )

        date = datetime.date.today().strftime("%Y-%m-%d")
        before_header = "## [Unreleased]\n\n"
        after_header = f"## [Unreleased]\n\n## v{after_version} ({date})\n\n"
        _search_and_replace("CHANGELOG.md", before_header, after_header)


def _pipenv_install(session: nox.Session) -> None:
    session.install("pipenv")
    session.run(
        "pipenv",
        "install",
        "--dev",
        env={"PIPENV_VERBOSITY": "-1"},
        *session.posargs,
    )


def _search_and_replace(file, search, replace) -> None:
    with open(file, "r+", encoding="utf8") as f:
        content = f.read()
        content = content.replace(search, replace)
        f.seek(0)
        f.write(content)
        f.truncate()
